Package: CexoR
Version: 1.48.0
Date: 2025-07-22
Type: Package
Title: An R package to uncover high-resolution protein-DNA interactions
        in ChIP-exo replicates
Authors@R: 
    c(person(given = "Pedro",
           family = "Madrigal",
           role = c("aut", "cre"),
           email = "pmadrigal@ebi.ac.uk",
           comment = c(ORCID = "0000-0003-1959-8199")
      )
    )
Description: Strand specific peak-pair calling in ChIP-exo replicates.
        The cumulative Skellam distribution function
        is used to detect significant normalised count
        differences of opposed sign at each DNA strand (peak-pairs).
        Then, irreproducible discovery rate for overlapping peak-pairs across
        biological replicates is computed.
Depends: R (>= 4.2.0), S4Vectors, IRanges
Imports: Rsamtools, Seqinfo, GenomicRanges, rtracklayer, idr,
        RColorBrewer, genomation
Suggests: RUnit, BiocGenerics, BiocStyle, knitr, rmarkdown
License: Artistic-2.0 | GPL-2 + file LICENSE
biocViews: FunctionalGenomics, Sequencing, Coverage, ChIPSeq,
        PeakDetection
URL: https://github.com/pmb59/CexoR
BugReports: https://github.com/pmb59/CexoR/issues
git_url: https://git.bioconductor.org/packages/CexoR
git_branch: RELEASE_3_22
git_last_commit: 2ba0990
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 03:04:12 UTC; biocbuild
Author: Pedro Madrigal [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1959-8199>)
Maintainer: Pedro Madrigal <pmadrigal@ebi.ac.uk>
