# CexoR

[![Anaconda-Server Badge](https://anaconda.org/bioconda/bioconductor-cexor/badges/downloads.svg)](https://anaconda.org/bioconda/bioconductor-cexor)
[![install with bioconda](https://img.shields.io/badge/install%20with-bioconda-brightgreen.svg?style=flat)](http://bioconda.github.io/recipes/bioconductor-cexor/README.html)
[![DOI:10.14806/ej.21.0.837](https://zenodo.org/badge/DOI/10.14806/ej.21.0.837.svg)](https://doi.org/10.14806/ej.21.0.837)
